#include "stdafx.h"

cplx AS_15_3_4_polynom (cplx a, cplx b, cplx c, cplx z)
{
  cplx result;
  result = pow(1.0 - z, c - a - b) * polynom_2F1(c - b, c - a, c, z, 1111);
  return(result);
}

// linear transforms (not as in Abramowitz & Stegun):
// transformed = GAMMA(c) * PI/sin(PI*z) * g(a,b,c,z)
// compare the proofs in Lebedev's book
// control the needed parameter configuration through the main program

cplx g_15_3_6 (cplx a, cplx b, cplx c, cplx z)
{
	cplx w, mu, f1, f2, g1, g2;

  w =  (1.0 - z);
	mu = (c - a - b);
  f1 = invGAMMA(b + mu) * invGAMMA(a + mu);
  f2 = pow(w, mu) * invGAMMA(a) * invGAMMA(b);
	g1 = G_2F1(a, b, -mu + 1.0, w) * invGAMMA(-mu + 1.0) ;
	g2 = G_2F1(a + mu, b + mu, 1.0 + mu, w) * invGAMMA(1.0 + mu);
	return f1 * g1 - f2 * g2;
}

cplx g_15_3_7 (cplx a, cplx b, cplx c, cplx z)
{
  cplx w, mu, f1, f2, g1, g2;

  w =  (1.0 / z);
  mu = (b - a);
  f1 = pow(-z, -a) * invGAMMA(c - a) * invGAMMA(b);
	f2 = pow(-z, -b) * invGAMMA(a) * invGAMMA(c - b);
	g1 = G_2F1(a, 1.0 + a - c, -mu + 1.0, w) * invGAMMA(-mu + 1.0);
	g2 = G_2F1(b, b - c + 1.0, mu + 1.0, w) * invGAMMA(mu + 1.0);
	return ( f1 * g1 - f2 *g2 );
}

cplx g_15_3_9 (cplx a, cplx b, cplx c, cplx z)
{
  cplx w, mu, f1, f2, g1, g2;

	w =  (z - 1.0);
	w =  (w / z);		// w:= (z-1)/z;
  mu = (c - a - b);
	f1 = pow(z, -a) * invGAMMA(b + mu) * invGAMMA(a + mu);
	f2 = pow(1.0 - z, mu) * pow(z, -a - mu) * invGAMMA(b) * invGAMMA(a);
	g1 = G_2F1(a, 1.0 - b - mu, -mu + 1.0, w) * invGAMMA(-mu + 1.0);
	g2 = G_2F1(a + mu, 1.0 - b, mu + 1.0, w) * invGAMMA(mu + 1.0);
	return ( f1 * g1 - f2 *g2 );
}

// ###############################################################################################
// exceptional parameters: numerical differentiation
// this avoids the usual series in 4 Psi functions given in Abramowitz & Stegun

cplx exceptional_15_3_6 (
  cplx a,
  cplx b,
  cplx C, // C
  cplx z)
{
  int m;
  double mu;
  cplx c;
  cplx result;
  cplx h;
  cplx dg;
	cplx x;

  mu = (real(C - a - b));
  m = roundToInteger(mu);
  c = (cplx) ( (double)m + a + b);
	x = c;

	h = STEPSIZE * theI;
	dg =
		0.3571428571428571e-2	*g_15_3_6(a,b,x-4.0*h,z)
		-0.3809523809523810e-1*g_15_3_6(a,b,x-3.0*h,z)
		+0.2000000000000000		*g_15_3_6(a,b,x-2.0*h,z)
		-0.8000000000000000		*g_15_3_6(a,b,x-1.0*h,z)
		+0.8000000000000000		*g_15_3_6(a,b,x+1.0*h,z)
		-0.2000000000000000		*g_15_3_6(a,b,x+2.0*h,z)
		+0.3809523809523810e-1*g_15_3_6(a,b,x+3.0*h,z)
		-0.3571428571428571e-2*g_15_3_6(a,b,x+4.0*h,z);
	dg = dg / h;

  result = GAMMA(c) * dg;
 	if (isOdd(m) == 1)
    result = -result;
	return(result);
}

cplx exceptional_15_3_7 (
  cplx a,
  cplx B, // B
  cplx c,
  cplx z)
{
  int m;
  double mu;
  cplx b;
  cplx result;
  cplx h;
  cplx dg;
	cplx x;

  mu =(real(B - a));
  m = roundToInteger(mu);
  b = (cplx) ( (double)m + a);
	x = b;

	h = STEPSIZE * theI;
	dg =
		0.3571428571428571e-2	*g_15_3_7(a,x-4.0*h,c,z)
		-0.3809523809523810e-1*g_15_3_7(a,x-3.0*h,c,z)
		+0.2000000000000000		*g_15_3_7(a,x-2.0*h,c,z)
		-0.8000000000000000		*g_15_3_7(a,x-1.0*h,c,z)
		+0.8000000000000000		*g_15_3_7(a,x+1.0*h,c,z)
		-0.2000000000000000		*g_15_3_7(a,x+2.0*h,c,z)
		+0.3809523809523810e-1*g_15_3_7(a,x+3.0*h,c,z)
		-0.3571428571428571e-2*g_15_3_7(a,x+4.0*h,c,z);
	dg = dg / h;

  result = GAMMA(c) * dg;
 	if (isOdd(m) == 1)
    result = -result;
	return(result);
}

cplx exceptional_15_3_9 (
  cplx a,
  cplx b,
  cplx C, // C
  cplx z)
{
  int m;
  double mu;
  cplx c;
  cplx result;
  cplx h;
  cplx dg;
	cplx x;

  mu = real(C - a - b);
	m = roundToInteger(mu);
  c = (cplx) ( (double)m + a + b);
	x = c;

	h = STEPSIZE * theI;
	dg =
		0.3571428571428571e-2	*g_15_3_9(a,b,x-4.0*h,z)
		-0.3809523809523810e-1*g_15_3_9(a,b,x-3.0*h,z)
		+0.2000000000000000		*g_15_3_9(a,b,x-2.0*h,z)
		-0.8000000000000000		*g_15_3_9(a,b,x-1.0*h,z)
		+0.8000000000000000		*g_15_3_9(a,b,x+1.0*h,z)
		-0.2000000000000000		*g_15_3_9(a,b,x+2.0*h,z)
		+0.3809523809523810e-1*g_15_3_9(a,b,x+3.0*h,z)
		-0.3571428571428571e-2*g_15_3_9(a,b,x+4.0*h,z);
	dg = dg / h;

  result = GAMMA(c) * dg;
 	if (isOdd(m) == 1)
    result = -result;
	return(result);
}

// ###############################################################################################
// non-exceptional parameters

cplx AS_15_3_6_Leb (cplx a, cplx b, cplx c, cplx z)
{
  if (isNumericalInteger(c - a - b) == 1)
    return(exceptional_15_3_6(a, b, c, z));
  return(invSINN(c - a - b) * GAMMA(c) * g_15_3_6(a,b,c,z));
}

cplx AS_15_3_7_Leb (cplx a, cplx b, cplx c, cplx z)
{
  if (isNumericalInteger(a - b) == 1)
    return(exceptional_15_3_7(a, b, c, z));
	return(invSINN(b - a) * GAMMA(c) * g_15_3_7(a,b,c,z));
}

cplx AS_15_3_9_Leb (cplx a, cplx b, cplx c, cplx z)
{
  if (isNumericalInteger(c - a - b) == 1)
    return(exceptional_15_3_9(a, b, c, z));
	return(invSINN(c - a - b) * GAMMA(c) * g_15_3_9(a,b,c,z));
}