#include "stdafx.h"

// round to nearest integer
int roundToInteger (double x)
{
	if (0.0 < x)
		return (int)floor(x + 0.5);
	return (int)ceil(x - 0.5);
}

int isOdd (int m)
{
	if (m % 2 == 0)
    return (0);
  else
    return (1);
}

// fcmp in GSL
int areClose (const double x1, const double x2, const double epsilon)
{
  int exponent;
  double delta, difference, maxXY, dummy;

  /* Find exponent of largest absolute value */
  maxXY = (fabs (x1) > fabs (x2)) ? x1 : x2;
  dummy = frexp (maxXY, &exponent);

  /* Form a neighborhood of size  2 * delta */
  delta = ldexp (epsilon, exponent);
  difference = x1 - x2;

  if (difference > delta)       /* x1 > x2 */
      return 0;
  else if (difference < -delta) /* x1 < x2 */
      return 0;
  else                          /* -delta <= difference <= delta */
      return 1;                 /* x1 ~=~ x2 */
}

int areEqual (cplx a, cplx b)
{
	if ( real(a) == real(b) && 
        imag(a) == imag(b) )
    return(1);

  if ( areClose(real(a), real(b), DBL_EPSILON) && 
        areClose(imag(a), imag(b), DBL_EPSILON) )
    return(1);
  
	return(0);
}

int isNumericalInteger (cplx z)
{
  double x;
	
  if ( fabs(imag(z)) < DBL_EPSILON)
  {
		x = fabs(real(z));
		if ( fabs(x - (int)(x)) <= DBL_EPSILON ) // this is cutting off decimal part
      return(1);
  }
  
	return(0);
}

int isNumericalNaturalNumber (cplx z)
{
  if ( (- 10*DBL_EPSILON <= real(z)) && (isNumericalInteger(z) == 1) )
		return(1);
  return(0);
}


cplx invSINN (cplx z)	// = PI / sin( PI * z)
{
	cplx z_red;
	double x,y, x_red, t;
	int m;
	cplx result;

	x = real(z); y = imag(z);

	if ( x == 0.0 )
	{
		t = PI * y;
		result = - PI / sinh(PI * y);
		result = 0.0 + real(result) * theI;
		return (result); // purely imaginary
	}

	// range reduction
	m = roundToInteger(x);

	// -0.5 < x < 0.5, x <> 0
	if ( m == 0 )
	{
		result =  PI / sin( PI * z);
		if (y == 0.0 )
			result = real(result) + 0.0*theI;	// clean up real case
		return (result);
	}

	x_red = (x - (double)m);
	z_red = x_red + y *theI;

	result = PI / sin( PI * z_red);
	if ( isOdd(m) == 1 )
		result = - result;
	
	if (y == 0.0 )	// purely real
		result = real(result) + 0.0*theI;
	return (result);
}
